/*---------------------------------------------------------------------------------
	Copyright 2013- j2jlab, Inc. All Rights Reserved.		by ODIN
---------------------------------------------------------------------------------*/

//precision mediump float;
varying mediump vec4	finColor;
varying mediump _uvvec  finTexCoord;
varying highp vec4	finViewVec;
varying mediump vec4 finTanDiffPos;	/*for Dynamic light */

varying highp vec3		finTanMat0;
varying highp vec3		finTanMat1;
varying highp vec3		finTanMat2;

varying highp vec3		finLocal;
varying lowp vec3		finNormal;

uniform lowp vec3  LightDir; 
uniform lowp vec3  LightAmbient;
uniform lowp vec3  LightColor;

uniform highp vec4 FogPara;
#define FogStart	FogPara.r
#define FogEnd		FogPara.g
#define FogDensity	FogPara.b
#define FogLimit	FogPara.a

#define FogIntensity	finViewVec.w
#ifdef _Gf_UNLIT
#define gAlpha	1.0
#define AlphaMaskValue	0.24
#else
uniform lowp float AlphaMaskValue;
uniform lowp float gAlpha;
#endif

uniform lowp vec3 FogColor;


void main (void)
{
	lowp vec3 ambient = vec3(0);
	lowp vec3 diffuse = vec3(1);
	lowp vec3 lightmap = vec3(1);
	lowp float NdotL = 1.0;
	lowp vec3  specComp = vec3(0);
	lowp vec3 FragColor;
	mediump vec3 normal;

	lightmap = _LIGHTMAP_COMPONENT * 2.0;
	gl_FragColor.a = _OPASITY_COMPONENT * finColor.a * gAlpha;
#ifdef _Gf_USE_ALPHA_TEST		/* ׽Ʈ ӵ ƸԴ°Ŷ  ̴  Ѵ.*/
 	if( gl_FragColor.a < AlphaMaskValue )
	{
 		discard;
	}
#endif

#ifdef _Gf_USE_TANGENT_VERTEX_STRUCT
	#ifdef _Gf_USE_NORMAL_MAP
 		normal = _NORMAL_COMPONENT;
 		highp mat3 tangentSpaceXform = mat3(finTanMat0, finTanMat1, finTanMat2);
 		normal = tangentSpaceXform*normal;
		normal = normalize(normal);
	#else
		normal = normalize(finNormal);
	#endif

	NdotL = GetNdotL(LightDir, normal);
	lightmap *= LightColor;
#else
	normal = normalize(finNormal);

	#ifdef _Gf_PER_PIXEL_ATTN
 		NdotL = GetNdotL(LightDir, normal);
		lightmap *= LightColor;
	#endif
#endif



	ambient = _AMBIENT_COMPONENT;

#ifdef _Gf_USE_SPECULAR
	lowp vec3 normalizedHalfAngle = normalize( finViewVec.xyz + LightDir );
	specComp = pow( max( dot(normal,normalizedHalfAngle), 0.0),  _GLOSSNESS_COMPONENT ) * vSpecColor  * _SPEC_LEVEL_COMPONENT;
#endif
	diffuse = GammaCorrect( _DIFFUSE_COMPONENT );
	lowp vec3 self_ill= _SELF_ILLUMINATION_COMPONENT;
	
#ifdef _Gf_USE_RIMLIGHT
	self_ill += ( vec3(1.0,0.0,0.0) * ( GetFresnelTerm(finViewVec.xyz, normal, 3.0 ) * 1.0 ) );
#endif

#ifdef _Gf_PER_PIXEL_ATTN
	diffuse *= finColor.rgb;
#else
	lightmap *= finColor.rgb;
#endif
	FragColor.xyz = diffuse * ( NdotL * lightmap + ambient ) + specComp * lightmap + self_ill;

#ifdef _Gf_USE_TANGENT_VERTEX_STRUCT
	lowp vec3 add_diffuse = vec3(0.0,0.0,0.0);
	lowp vec3 add_spec = vec3(0.0,0.0,0.0);
	//߰Ʈ
	{
		lowp vec3 lgt_density2 = AddLightColor.rgb;
		mediump vec3 addLightNormalWS = normalize(finTanDiffPos.xyz);
		mediump float attn;
	#ifdef _Gf_PER_PIXEL_ATTN
		attn = GetAttnFromLight((AddLightPos.xyz - finLocal),_GetAddLightAttnStart,_GetAddLightAttnEnd);
	#else
		attn = finTanDiffPos.w;
	#endif
 		NdotL = GetNdotL(addLightNormalWS, normal);
		mediump vec3 middle = NdotL * lgt_density2 * diffuse;
		add_diffuse += attn * middle;

	#ifdef _Gf_USE_SPECULAR
 		normalizedHalfAngle = normalize( finViewVec.xyz + addLightNormalWS );
 		lowp vec3 spec = pow( max( dot(normal,normalizedHalfAngle), 0.0),  _GLOSSNESS_COMPONENT ) * vSpecColor * _SPEC_LEVEL_COMPONENT;
 		spec *= AddLightColor.rgb*attn;
 		add_spec += spec;
	#endif
	}
	FragColor.xyz += add_diffuse + add_spec;
#else	//_Gf_USE_TANGENT_VERTEX_STRUCT
	#ifdef _Gf_PER_PIXEL_ATTN
		lowp vec3 add_diffuse = vec3(0.0,0.0,0.0);
		lowp vec3 add_spec = vec3(0.0,0.0,0.0);
		//߰Ʈ
		{
 			lowp vec3 lgt_density2 = AddLightColor.rgb;
 			mediump vec3 DiffPos = (AddLightPos.xyz - finLocal);
 			mediump vec3 addLightNormalWS = normalize(DiffPos);
 			mediump float attn;
 			attn = GetAttnFromLight(DiffPos,_GetAddLightAttnStart,_GetAddLightAttnEnd);
	 		NdotL = GetNdotL(addLightNormalWS, normal);
 			mediump vec3 middle = NdotL * lgt_density2 * diffuse;
 			add_diffuse += attn * middle;
			#ifdef _Gf_USE_SPECULAR
 				normalizedHalfAngle = normalize( finViewVec.xyz + addLightNormalWS );
 				lowp vec3 spec = pow( max( dot(normal,normalizedHalfAngle), 0.0),  _GLOSSNESS_COMPONENT ) * vSpecColor * _SPEC_LEVEL_COMPONENT;
 				spec *= AddLightColor.rgb*attn;
 				add_spec += spec;
			#endif
		}
		FragColor.xyz += add_diffuse + add_spec;
	#endif	//_Gf_PER_PIXEL_ATTN
#endif	//not _Gf_USE_TANGENT_VERTEX_STRUCT 

#ifndef _Gf_UNLIT
	//fog.
//	if( FogEnd != FogStart )
	{
		highp float fogIntensity = (FogIntensity-FogStart)/(FogEnd-FogStart);
 		fogIntensity = clamp(fogIntensity, 0.0, 1.0) * FogLimit;
		FragColor = mix(FragColor,FogColor,vec3(fogIntensity));
	}
#endif

	gl_FragColor.xyz = FragColor;
}
